DEFAULT_VECTORE_STORE_TYPE = 'pinecone'
DEFAULT_INDEX_NAME = 'quickstart'
DEFAULT_LLM_CLASS_NAME = 'OpenAI'

DEFAULT_RELATIVE_DOCS_PATH = 'docs'

DEFAULT_CHUNK_SIZE = 1024
DEFAULT_CHUNK_OVERLAP = 20
DEFAULT_CONTEXT_WINDOW = 4096

DEFAULT_MAX_TOKENS = 1024
DEFAULT_OPENAI_MODEL = 'gpt-3.5-turbo'
DEFAULT_PALM_MODEL = 'models/text-bison-001'
DEFAULT_ANYSCALE_MODEL = 'meta-llama/Llama-2-70b-chat-hf'

WEBPAGE_READER_TIMEOUT = 3
